#include "Move Utility.h"

Move_Envelope Move_Utility::determine_move(int type, int id, int cells[])
{
	if(type == KING) { return move_king(id, cells); }
	else if(type == QUEEN) { return move_queen(id, cells); }
	else if(type == ROOK) { return move_rook(id, cells); }
	else if(type == BISHOP) { return move_bishop(id, cells); }
	else if(type == KNIGHT) { return move_knight(id, cells); }
	else if(type == PAWN) { return move_pawn(id, cells); }
	else { 	Move_Envelope out = Move_Envelope(); return out; }
}

Move_Envelope Move_Utility::move_pawn(int id, int cells[])
{
	bool top = is_top(id);
	bool bottom = is_bottom(id);
	bool left = is_left(id);
	bool right = is_right(id);

	Move_Envelope envelope = Move_Envelope();

	//White moves up, black moves down
	if(cells[id] >= WHITE_KING && cells[id] <= WHITE_PAWN8)
	{
		if(!top && !is_friendly_piece(id, id-8, cells) && !is_enemy_piece(id, id-8, cells))
		{
			Move m_top = Move();
			m_top.add_cell(id-8);
			envelope.add_move(m_top);
		}

		//Initial move - can move two places
		if(!top && id >= 48 && id <= 55 && !is_friendly_piece(id, id-8, cells) && !is_enemy_piece(id, id-8, cells) && !is_friendly_piece(id, id-16, cells) && !is_enemy_piece(id, id-16, cells))
		{
			Move m_top = Move();
			//m_top.add_cell(id-8);
			m_top.add_cell(id-16);
			envelope.add_move(m_top);	
		}

		if(!top && !left && is_enemy_piece(id, id-9, cells))
		{
			Move m_top = Move();
			m_top.add_cell(id-9);
			m_top.isCapture = true;
			envelope.add_move(m_top);
		}

		if(!top && !right && is_enemy_piece(id, id-7, cells))
		{
			Move m_right = Move();
			m_right.add_cell(id-7);		
			m_right.isCapture = true;
			envelope.add_move(m_right);
		}
	}
	else
	{
		if(!bottom && !is_friendly_piece(id, id+8, cells) && !is_enemy_piece(id, id+8, cells))
		{
			Move m_bottom = Move();
			m_bottom.add_cell(id+8);		
			if(is_enemy_piece(id, id+8, cells)) m_bottom.isCapture = true;
			envelope.add_move(m_bottom);
		}

		//Initial move - can move two places
		if(!bottom && id >= 8 && id <= 15 && !is_friendly_piece(id, id+8, cells) && !is_enemy_piece(id, id+8, cells) && !is_friendly_piece(id, id+16, cells) && !is_enemy_piece(id, id+16, cells))
		{
			Move m_top = Move();
			//m_top.add_cell(id+8);
			m_top.add_cell(id+16);
			envelope.add_move(m_top);	
		}

		if(!bottom && !left && is_enemy_piece(id, id+7, cells))
		{
			Move m_bottom = Move();
			m_bottom.add_cell(id+7);		
			m_bottom.isCapture = true;
			envelope.add_move(m_bottom);
		}

		if(!bottom && !right && is_enemy_piece(id, id+9, cells))
		{
			Move m_left = Move();
			m_left.add_cell(id+9);
			m_left.isCapture = true;
			envelope.add_move(m_left);
		}
	}

	return envelope;
}

Move_Envelope Move_Utility::move_king(int id, int cells[])
{
	bool top = is_top(id);
	bool bottom = is_bottom(id);
	bool left = is_left(id);
	bool right = is_right(id);

	Move_Envelope envelope = Move_Envelope();
	
	//King can only move one place in any of 8 directions
	recurse_up(id, id, 1, 0, cells, envelope);
	recurse_up_left(id, id, 1, 0, cells, envelope);
	recurse_left(id, id, 1, 0, cells, envelope);
	recurse_down_left(id, id, 1, 0, cells, envelope);
	recurse_down(id, id, 1, 0, cells, envelope);
	recurse_down_right(id, id, 1, 0, cells, envelope);
	recurse_right(id, id, 1, 0, cells, envelope);
	recurse_up_right(id, id, 1, 0, cells, envelope);

	return envelope;
}

Move_Envelope Move_Utility::move_queen(int id, int cells[])
{
	Move_Envelope envelope = Move_Envelope();
	
	//King can only move any number of places (max 7) in any of 8 directions
	recurse_up(id, id, 7, 0, cells, envelope);
	recurse_up_left(id, id, 7, 0, cells, envelope);
	recurse_left(id, id, 7, 0, cells, envelope);
	recurse_down_left(id, id, 7, 0, cells, envelope);
	recurse_down(id, id, 7, 0, cells, envelope);
	recurse_down_right(id, id, 7, 0, cells, envelope);
	recurse_right(id, id, 7, 0, cells, envelope);
	recurse_up_right(id, id, 7, 0, cells, envelope);

	return envelope;
}

Move_Envelope Move_Utility::move_rook(int id, int cells[])
{
	Move_Envelope envelope = Move_Envelope();
	
	//Rook can only move any number of places (max 7) in up-down or left-right directions
	recurse_up(id, id, 7, 0, cells, envelope);
	recurse_left(id, id, 7, 0, cells, envelope);
	recurse_down(id, id, 7, 0, cells, envelope);
	recurse_right(id, id, 7, 0, cells, envelope);

	return envelope;
}

Move_Envelope Move_Utility::move_bishop(int id, int cells[])
{
	Move_Envelope envelope = Move_Envelope();
	
	//Bishop can only move any number of places (max 7) in diagonal directions
	recurse_up_left(id, id, 7, 0, cells, envelope);
	recurse_down_left(id, id, 7, 0, cells, envelope);
	recurse_down_right(id, id, 7, 0, cells, envelope);
	recurse_up_right(id, id, 7, 0, cells, envelope);

	return envelope;
}

Move_Envelope Move_Utility::move_knight(int id, int cells[])
{
	bool top_outer = (id >= 16) ? true : false;
	bool top_inner = (id >= 8) ? true : false;
	bool bottom_outer = (id <= 47) ? true : false;
	bool bottom_inner = (id <= 55) ? true : false;
	bool left_outer = (id%8 != 0 && (id-1)%8 != 0) ? true : false;
	bool left_inner = (id%8 != 0) ? true : false;
	bool right_outer = ((id+1)%8 != 0 && (id+2)%8 != 0) ? true : false;
	bool right_inner = ((id+1)%8 != 0) ? true : false;

	Move_Envelope envelope = Move_Envelope();
	if(top_outer && left_inner && !is_friendly_piece(id, id-17,cells))
	{
		Move m = Move();
		//m.add_cell(id-8);
		//m.add_cell(id-16);
		m.add_cell(id-17);
		if(is_enemy_piece(id, id-17, cells)) m.isCapture = true;
		envelope.add_move(m);
	}

	if(top_outer && right_inner && !is_friendly_piece(id, id-15,cells))
	{
		Move m = Move();
		//m.add_cell(id-8);
		//m.add_cell(id-16);
		m.add_cell(id-15);
		if(is_enemy_piece(id, id-15, cells)) m.isCapture = true;
		envelope.add_move(m);
	}

	if(top_inner && right_outer && !is_friendly_piece(id, id-6,cells))
	{
		Move m = Move();
		//m.add_cell(id-8);
		//m.add_cell(id-7);
		m.add_cell(id-6);
		if(is_enemy_piece(id, id-6, cells)) m.isCapture = true;
		envelope.add_move(m);
	}

	if(bottom_inner && right_outer && !is_friendly_piece(id, id+10,cells))
	{
		Move m = Move();
		//m.add_cell(id+8);
		//m.add_cell(id+9);
		m.add_cell(id+10);
		if(is_enemy_piece(id, id+10, cells)) m.isCapture = true;
		envelope.add_move(m);
	}

	if(bottom_outer && right_inner && !is_friendly_piece(id, id+17,cells))
	{
		Move m = Move();
		//m.add_cell(id+8);
		//m.add_cell(id+16);
		m.add_cell(id+17);
		if(is_enemy_piece(id, id+17, cells)) m.isCapture = true;
		envelope.add_move(m);
	}

	if(bottom_outer && left_inner && !is_friendly_piece(id, id+15,cells))
	{
		Move m = Move();
		//m.add_cell(id+8);
		//m.add_cell(id+16);
		m.add_cell(id+15);
		if(is_enemy_piece(id, id+15, cells)) m.isCapture = true;
		envelope.add_move(m);
	}

	if(bottom_inner && left_outer && !is_friendly_piece(id, id+6,cells))
	{
		Move m = Move();
		//m.add_cell(id+8);
		//m.add_cell(id+7);
		m.add_cell(id+6);
		if(is_enemy_piece(id, id+6, cells)) m.isCapture = true;
		envelope.add_move(m);
	}

	if(top_inner && left_outer && !is_friendly_piece(id, id-10,cells))
	{
		Move m = Move();
		//m.add_cell(id-8);
		//m.add_cell(id-9);
		m.add_cell(id-10);
		if(is_enemy_piece(id, id-10, cells)) m.isCapture = true;
		envelope.add_move(m);
	}

	return envelope;
}


bool Move_Utility::recurse_up(int src,  int current_position, int step_count, int current_step, int cells[], Move_Envelope &envelope)
{
	if(step_count == current_step) return false;
	if(is_top(current_position)) return false;
	if(is_friendly_piece(src, current_position-8,cells)) return false;

	Move m_up = Move();
	m_up.add_cell(current_position-8);
	if(is_enemy_piece(src, current_position-8, cells))
	{
		m_up.isCapture = true;
		envelope.add_move(m_up);
		return false;
	}
	else envelope.add_move(m_up);

	return recurse_up(src, current_position-8, step_count, current_step+1, cells, envelope);
}

bool Move_Utility::recurse_down(int src,  int current_position, int step_count, int current_step, int cells[], Move_Envelope &envelope)
{
	if(step_count == current_step) return false;
	if(is_bottom(current_position)) return false;
	if(is_friendly_piece(src, current_position+8,cells)) return false;

	Move m_down = Move();
	m_down.add_cell(current_position+8);
	if(is_enemy_piece(src, current_position+8, cells))
	{
		m_down.isCapture = true;
		envelope.add_move(m_down);
		return false;
	}
	else envelope.add_move(m_down);

	return recurse_down(src, current_position+8, step_count, current_step+1, cells, envelope);
}

bool Move_Utility::recurse_left(int src,  int current_position, int step_count, int current_step, int cells[], Move_Envelope &envelope)
{
	if(step_count == current_step) return false;
	if(is_left(current_position)) return false;
	if(is_friendly_piece(src, current_position-1,cells)) return false;

	Move m_left = Move();
	m_left.add_cell(current_position-1);
	if(is_enemy_piece(src, current_position-1, cells))
	{
		m_left.isCapture = true;
		envelope.add_move(m_left);
		return false;
	}
	else envelope.add_move(m_left);

	return recurse_left(src, current_position-1, step_count, current_step+1, cells, envelope);
}

bool Move_Utility::recurse_right(int src,  int current_position, int step_count, int current_step, int cells[], Move_Envelope &envelope)
{
	if(step_count == current_step) return false;
	if(is_right(current_position)) return false;
	if(is_friendly_piece(src, current_position+1,cells)) return false;

	Move m_right = Move();
	m_right.add_cell(current_position+1);
	if(is_enemy_piece(src, current_position+1, cells))
	{
		m_right.isCapture = true;
		envelope.add_move(m_right);
		return false;
	}
	else envelope.add_move(m_right);

	return recurse_right(src, current_position+1, step_count, current_step+1, cells, envelope);
}

bool Move_Utility::recurse_up_left(int src,  int current_position, int step_count, int current_step, int cells[], Move_Envelope &envelope)
{
	if(step_count == current_step) return false;
	if(is_top(current_position) || is_left(current_position)) return false;
	if(is_friendly_piece(src, current_position-9,cells)) return false;

	Move m_up_left = Move();
	m_up_left.add_cell(current_position-9);
	if(is_enemy_piece(src, current_position-9, cells))
	{
		m_up_left.isCapture = true;
		envelope.add_move(m_up_left);
		return false;
	}
	else envelope.add_move(m_up_left);

	return recurse_up_left(src, current_position-9, step_count, current_step+1, cells, envelope);
}

bool Move_Utility::recurse_down_left(int src,  int current_position, int step_count, int current_step, int cells[], Move_Envelope &envelope)
{
	if(step_count == current_step) return false;
	if(is_bottom(current_position) || is_left(current_position)) return false;
	if(is_friendly_piece(src, current_position+7,cells)) return false;

	Move m_down_left = Move();
	m_down_left.add_cell(current_position+7);
	if(is_enemy_piece(src, current_position+7, cells))
	{
		m_down_left.isCapture = true;
		envelope.add_move(m_down_left);
		return false;
	}
	else envelope.add_move(m_down_left);

	return recurse_down_left(src, current_position+7, step_count, current_step+1, cells, envelope);
}

bool Move_Utility::recurse_down_right(int src,  int current_position, int step_count, int current_step, int cells[], Move_Envelope &envelope)
{
	if(step_count == current_step) return false;
	if(is_bottom(current_position) || is_right(current_position)) return false;
	if(is_friendly_piece(src, current_position+9,cells)) return false;

	Move m_down_right = Move();
	m_down_right.add_cell(current_position+9);
	if(is_enemy_piece(src, current_position+9, cells))
	{
		m_down_right.isCapture = true;
		envelope.add_move(m_down_right);
		return false;
	}
	else envelope.add_move(m_down_right);

	return recurse_down_right(src, current_position+9, step_count, current_step+1, cells, envelope);
}

bool Move_Utility::recurse_up_right(int src,  int current_position, int step_count, int current_step, int cells[], Move_Envelope &envelope)
{
	if(step_count == current_step) return false;
	if(is_top(current_position) || is_right(current_position)) return false;
	if(is_friendly_piece(src, current_position-7,cells)) return false;

	Move m_up_right = Move();
	m_up_right.add_cell(current_position-7);
	if(is_enemy_piece(src, current_position-7, cells))
	{
		m_up_right.isCapture = true;
		envelope.add_move(m_up_right);
		return false;
	}
	else envelope.add_move(m_up_right);

	return recurse_up_right(src, current_position-7, step_count, current_step+1, cells, envelope);
}


bool Move_Utility::is_friendly_piece(int src, int dest, int cells[]) 
{ 
	if(cells[src] == NONE || cells[dest] == NONE) return false;
	else
	{
		bool is_white = (cells[src] >= WHITE_KING && cells[src] <= WHITE_PAWN8) ? true : false;
		if(is_white && cells[dest] >= WHITE_KING && cells[dest] <= WHITE_PAWN8) return true;
		else if(!is_white && cells[dest] >= BLACK_KING && cells[dest] <= BLACK_PAWN8) return true;
		else return false;
	}
}

bool Move_Utility::is_enemy_piece(int src, int dest, int cells[]) 
{ 
	if(cells[src] == NONE || cells[dest] == NONE) return false;
	else
	{
		bool is_white = (cells[src] >= WHITE_KING && cells[src] <= WHITE_PAWN8) ? true : false;
		if(!is_white && cells[dest] >= WHITE_KING && cells[dest] <= WHITE_PAWN8) return true;
		else if(is_white && cells[dest] >= BLACK_KING && cells[dest] <= BLACK_PAWN8) return true;
		else return false;
	}
}

bool Move_Utility::is_left(int cell) { if(cell == 0 || cell%8 == 0) return true; else return false; }
bool Move_Utility::is_right(int cell) { if((cell+1)%8 == 0) return true; else return false; }
bool Move_Utility::is_top(int cell) { if(cell >= 0 && cell <= 7) return true; else return false; }
bool Move_Utility::is_bottom(int cell) { if(cell >= 56 && cell <= 63) return true; else return false; }
