#ifndef MOVE_UTILITY_H
#define MOVE_UTILITY_H
#include "Constants.h"
#include <vector>
#include <iostream>
using namespace std;

//A move from start to ending position
class Move 
{
public:
	bool isCapture;
	vector<int>cells;

	Move() { isCapture = false; cells.clear(); }
	void add_cell(int cell) { cells.push_back(cell); }
	void print(){ cout << "Move (" << isCapture <<") "; for(int i=0;i<(int)cells.size();i++){ cout << cells[i]; } cout << endl; }
};

//Contains a set of moves
class Move_Envelope
{
public:	
	vector<Move>moves;
	Move_Envelope(){ moves.clear(); }
	int move_count(){ return (int)moves.size(); }
	void add_move(Move move){ moves.push_back(move); }
	void clear_moves(){ moves.clear(); }

	void print() { cout << "++ Envelope ++\n"; for(int i=0;i<(int)moves.size();i++) moves[i].print(); }
};

class Move_Utility
{
public:
	static Move_Envelope determine_move(int type, int id, int cells[]);

private:

	static Move_Envelope move_king(int id, int cells[]);
	static Move_Envelope move_queen(int id, int cells[]);
	static Move_Envelope move_rook(int id, int cells[]);
	static Move_Envelope move_bishop(int id, int cells[]);
	static Move_Envelope move_knight(int id, int cells[]);
	static Move_Envelope move_pawn(int id, int cells[]);
	
	static bool is_friendly_piece(int src, int dest, int cells[]);
	static bool is_enemy_piece(int src, int dest, int cells[]);

	static bool is_left(int cell);
	static bool is_right(int cell);
	static bool is_top(int cell);
	static bool is_bottom(int cell);

	static bool recurse_up(int src, int current_position, int step_count, int current_step, int cells[], Move_Envelope &envelope);
	static bool recurse_down(int src, int current_position, int step_count, int current_step, int cells[], Move_Envelope &envelope);
	static bool recurse_left(int src, int current_position, int step_count, int current_step, int cells[], Move_Envelope &envelope);
	static bool recurse_right(int src, int current_position, int step_count, int current_step, int cells[], Move_Envelope &envelope);

	static bool recurse_up_left(int src, int current_position, int step_count, int current_step, int cells[], Move_Envelope &envelope);
	static bool recurse_down_left(int src, int current_position, int step_count, int current_step, int cells[], Move_Envelope &envelope);
	static bool recurse_up_right(int src, int current_position, int step_count, int current_step, int cells[], Move_Envelope &envelope);
	static bool recurse_down_right(int src, int current_position, int step_count, int current_step, int cells[], Move_Envelope &envelope);


};


#endif