#include "Constants.h"
#include "Move Utility.h"
#include "stdafx.h"
#include "sapi.h"
#include "sphelper.h" //for enumeration

#include <GL/glut.h>
#include <GL/glaux.h>
#include <iostream>
#include <vector>
using namespace std;

//Prototypes
void init_board();
void print_board();
void draw_board();
void draw_cell(int i, int j);
void draw_letter(int i);
void draw_number(int i);
void translate_to_cell(int id);
void translate_to_cell(int i, int j);
void set_moves();
void select_move();
void perform_move();
void reset_selected();
bool is_odd(int i, int j);

bool init_voices();
void speak(WCHAR theString[30], bool use_white_voice);
void speak_cell(int id, bool talk, WCHAR *theString, bool use_white_voice, bool isCapture);
void speak_cell_location(int id, WCHAR *theString);
void speak_cell_piece(int id, WCHAR *theString);
void query_move();

void gl_init();
void gl_draw();
void gl_special(int key, int x, int y);
void gl_key(unsigned char key, int x, int y); 

void CreateTexture(UINT textureArray[], LPSTR strFileName, int textureID);

int height = 850;
int width = 850;
int cell_size = 100;
int offset = 75;

int board[64];

UINT textures[40];	//Texture Data

Move_Envelope moves;
int selected_cell = 37;
int selected_cell_2 = -1;
bool show_moves = false;
int selected_move = -1;

bool is_white_move = true;
ISpVoice *black_voice; 
ISpVoice *white_voice; 

WCHAR string_to_speak[30];

bool isFirstTime = true;

int mode = TUTORIAL_MODE;
bool simulate_blindness = false;

/*----------------------------------------------------------------------------
 * Board Functions
 *----------------------------------------------------------------------------*/
//Initializes the board
void init_board()
{
	moves = Move_Envelope();

	//Set blank squares
	for(int i=0;i<64;i++) board[i] = NONE;

	//Black Pieces
	board[8] = BLACK_PAWN1;		board[9] = BLACK_PAWN2;
	board[10] = BLACK_PAWN3;	board[11] = BLACK_PAWN4;
	board[12] = BLACK_PAWN5;	board[13] = BLACK_PAWN6;
	board[14] = BLACK_PAWN7;	board[15] = BLACK_PAWN8;

	board[0] = BLACK_ROOK1;		board[1] = BLACK_KNIGHT1;
	board[2] = BLACK_BISHOP1;	board[3] = BLACK_QUEEN;
	board[4] = BLACK_KING;		board[5] = BLACK_BISHOP2;
	board[6] = BLACK_KNIGHT2;	board[7] = BLACK_ROOK2;

	//White Pieces
	board[48] = WHITE_PAWN1;	board[49] = WHITE_PAWN2;
	board[50] = WHITE_PAWN3;	board[51] = WHITE_PAWN4;
	board[52] = WHITE_PAWN5;	board[53] = WHITE_PAWN6;
	board[54] = WHITE_PAWN7;	board[55] = WHITE_PAWN8;

	board[56] = WHITE_ROOK1;	board[57] = WHITE_KNIGHT1;
	board[58] = WHITE_BISHOP1;	board[59] = WHITE_QUEEN;
	board[60] = WHITE_KING;		board[61] = WHITE_BISHOP2;
	board[62] = WHITE_KNIGHT2;	board[63] = WHITE_ROOK2;
	
	//Load textures
	//White Pieces
	CreateTexture(textures, ".\\images\\bking_dark.bmp", WHITE_KING_BLACK);
	CreateTexture(textures, ".\\images\\bking_light.bmp", WHITE_KING_WHITE);

	CreateTexture(textures, ".\\images\\bqueen_dark.bmp", WHITE_QUEEN_BLACK);
	CreateTexture(textures, ".\\images\\bqueen_light.bmp", WHITE_QUEEN_WHITE);

	CreateTexture(textures, ".\\images\\bbishop_dark.bmp", WHITE_BISHOP_BLACK);
	CreateTexture(textures, ".\\images\\bbishop_light.bmp", WHITE_BISHOP_WHITE);

	CreateTexture(textures, ".\\images\\brook_dark.bmp", WHITE_ROOK_BLACK);
	CreateTexture(textures, ".\\images\\brook_light.bmp", WHITE_ROOK_WHITE);

	CreateTexture(textures, ".\\images\\bknight_dark.bmp", WHITE_KNIGHT_BLACK);
	CreateTexture(textures, ".\\images\\bknight_light.bmp", WHITE_KNIGHT_WHITE);

	CreateTexture(textures, ".\\images\\bpawn_dark.bmp", WHITE_PAWN_BLACK);
	CreateTexture(textures, ".\\images\\bpawn_light.bmp", WHITE_PAWN_WHITE);

	//Black Pieces
	CreateTexture(textures, ".\\images\\rking_dark.bmp", BLACK_KING_BLACK);
	CreateTexture(textures, ".\\images\\rking_light.bmp", BLACK_KING_WHITE);

	CreateTexture(textures, ".\\images\\rqueen_dark.bmp", BLACK_QUEEN_BLACK);
	CreateTexture(textures, ".\\images\\rqueen_light.bmp", BLACK_QUEEN_WHITE);

	CreateTexture(textures, ".\\images\\rbishop_dark.bmp", BLACK_BISHOP_BLACK);
	CreateTexture(textures, ".\\images\\rbishop_light.bmp", BLACK_BISHOP_WHITE);

	CreateTexture(textures, ".\\images\\rrook_dark.bmp", BLACK_ROOK_BLACK);
	CreateTexture(textures, ".\\images\\rrook_light.bmp", BLACK_ROOK_WHITE);

	CreateTexture(textures, ".\\images\\rknight_dark.bmp", BLACK_KNIGHT_BLACK);
	CreateTexture(textures, ".\\images\\rknight_light.bmp", BLACK_KNIGHT_WHITE);

	CreateTexture(textures, ".\\images\\rpawn_dark.bmp", BLACK_PAWN_BLACK);
	CreateTexture(textures, ".\\images\\rpawn_light.bmp", BLACK_PAWN_WHITE);

	CreateTexture(textures, ".\\images\\a.bmp", A);
	CreateTexture(textures, ".\\images\\b.bmp", B);
	CreateTexture(textures, ".\\images\\c.bmp", C);
	CreateTexture(textures, ".\\images\\d.bmp", D);
	CreateTexture(textures, ".\\images\\e.bmp", E);
	CreateTexture(textures, ".\\images\\f.bmp", F);
	CreateTexture(textures, ".\\images\\g.bmp", G);
	CreateTexture(textures, ".\\images\\h.bmp", H);
	CreateTexture(textures, ".\\images\\one.bmp", ONE);
	CreateTexture(textures, ".\\images\\two.bmp", TWO);
	CreateTexture(textures, ".\\images\\three.bmp", THREE);
	CreateTexture(textures, ".\\images\\four.bmp", FOUR);
	CreateTexture(textures, ".\\images\\five.bmp", FIVE);
	CreateTexture(textures, ".\\images\\six.bmp", SIX);
	CreateTexture(textures, ".\\images\\seven.bmp", SEVEN);
	CreateTexture(textures, ".\\images\\eight.bmp", EIGHT);

}

//Prints the board to stdout
void print_board()
{
	for(int i=0;i<8;i++)
	{
		for(int j=0;j<8;j++) cout << board[(8*i)+j] << "\t";
		cout << endl;
	}
}

//Draws the board to screen
void draw_board()
{
	//Draw grid
	for(int i=0;i<8;i++)
	{
		glPushMatrix();
		glTranslatef(0,(cell_size*i),0);
		for(int j=0;j<8;j++)
		{
			glPushMatrix();
			glTranslatef((cell_size*j),0,0);
			if(i%2==0 && j%2==0) glColor3ub(255,255,213);
			else if(i%2==0 && j%2!=0) glColor3ub(82,115,98);
			else if(i%2!=0 && j%2==0) glColor3ub(82,115,98);
			else if(i%2!=0 && j%2!=0) glColor3ub(255,255,213);

				glPushMatrix();
				glTranslatef(offset, offset, 0);
				glScalef(cell_size, cell_size, 0);
				glutSolidCube(1);
				glPopMatrix();
			glPopMatrix();
		}
		glPopMatrix();
	}

	//Draw letters
	for(int i=0;i<8;i++)
	{
		glPushMatrix();
		glColor3f(1,1,1);
		glEnable(GL_TEXTURE_2D);

		if(i == 0) glBindTexture(GL_TEXTURE_2D, textures[A]);
		else if(i == 1) glBindTexture(GL_TEXTURE_2D, textures[B]);
		else if(i == 2) glBindTexture(GL_TEXTURE_2D, textures[C]);
		else if(i == 3) glBindTexture(GL_TEXTURE_2D, textures[D]);
		else if(i == 4) glBindTexture(GL_TEXTURE_2D, textures[E]);
		else if(i == 5) glBindTexture(GL_TEXTURE_2D, textures[F]);
		else if(i == 6) glBindTexture(GL_TEXTURE_2D, textures[G]);
		else if(i == 7) glBindTexture(GL_TEXTURE_2D, textures[H]);

		draw_letter(i);
		glDisable(GL_TEXTURE_2D);
		glPopMatrix();
	}

	//Draw numbers
	for(int i=0;i<8;i++)
	{
		glPushMatrix();
		glColor3f(1,1,1);
		glEnable(GL_TEXTURE_2D);

		if(i == 0) glBindTexture(GL_TEXTURE_2D, textures[ONE]);
		else if(i == 1) glBindTexture(GL_TEXTURE_2D, textures[TWO]);
		else if(i == 2) glBindTexture(GL_TEXTURE_2D, textures[THREE]);
		else if(i == 3) glBindTexture(GL_TEXTURE_2D, textures[FOUR]);
		else if(i == 4) glBindTexture(GL_TEXTURE_2D, textures[FIVE]);
		else if(i == 5) glBindTexture(GL_TEXTURE_2D, textures[SIX]);
		else if(i == 6) glBindTexture(GL_TEXTURE_2D, textures[SEVEN]);
		else if(i == 7) glBindTexture(GL_TEXTURE_2D, textures[EIGHT]);
		
		draw_number(i);
		glDisable(GL_TEXTURE_2D);
		glPopMatrix();
	}
	//Draw pieces
	for(int i=0;i<8;i++)
	{
		for(int j=0;j<8;j++)
		{
			int cell = (8*j)+i;

			glPushMatrix();
			glColor3f(1,1,1);
			glEnable(GL_TEXTURE_2D);

			if(board[cell] == BLACK_KING)
			{
				if(is_odd(i,j)) glBindTexture(GL_TEXTURE_2D, textures[BLACK_KING_WHITE]);
				else glBindTexture(GL_TEXTURE_2D, textures[BLACK_KING_BLACK]);
			}
			else if(board[cell] == BLACK_QUEEN)
			{
				if(is_odd(i,j)) glBindTexture(GL_TEXTURE_2D, textures[BLACK_QUEEN_WHITE]);
				else glBindTexture(GL_TEXTURE_2D, textures[BLACK_QUEEN_BLACK]);
			}
			else if(board[cell] == BLACK_BISHOP1 || board[cell] == BLACK_BISHOP2)
			{
				if(is_odd(i,j)) glBindTexture(GL_TEXTURE_2D, textures[BLACK_BISHOP_WHITE]);
				else glBindTexture(GL_TEXTURE_2D, textures[BLACK_BISHOP_BLACK]);
			}
			else if(board[cell] == BLACK_KNIGHT1 || board[cell] == BLACK_KNIGHT2)
			{
				if(is_odd(i,j)) glBindTexture(GL_TEXTURE_2D, textures[BLACK_KNIGHT_WHITE]);
				else glBindTexture(GL_TEXTURE_2D, textures[BLACK_KNIGHT_BLACK]);
			}
			else if(board[cell] == BLACK_ROOK1 || board[cell] == BLACK_ROOK2)
			{
				if(is_odd(i,j)) glBindTexture(GL_TEXTURE_2D, textures[BLACK_ROOK_WHITE]);
				else glBindTexture(GL_TEXTURE_2D, textures[BLACK_ROOK_BLACK]);
			}
			else if(board[cell] == BLACK_PAWN1 || board[cell] == BLACK_PAWN2 || board[cell] == BLACK_PAWN3 || board[cell] == BLACK_PAWN4 || board[cell] == BLACK_PAWN5 || board[cell] == BLACK_PAWN6 || board[cell] == BLACK_PAWN7 || board[cell] == BLACK_PAWN8)
			{
				if(is_odd(i,j)) glBindTexture(GL_TEXTURE_2D, textures[BLACK_PAWN_WHITE]);
				else glBindTexture(GL_TEXTURE_2D, textures[BLACK_PAWN_BLACK]);
			}

			//White Pieces
			else if(board[cell] == WHITE_KING)
			{
				if(is_odd(i,j)) glBindTexture(GL_TEXTURE_2D, textures[WHITE_KING_WHITE]);
				else glBindTexture(GL_TEXTURE_2D, textures[WHITE_KING_BLACK]);
			}
			else if(board[cell] == WHITE_QUEEN)
			{
				if(is_odd(i,j)) glBindTexture(GL_TEXTURE_2D, textures[WHITE_QUEEN_WHITE]);
				else glBindTexture(GL_TEXTURE_2D, textures[WHITE_QUEEN_BLACK]);
			}
			else if(board[cell] == WHITE_BISHOP1 || board[cell] == WHITE_BISHOP2)
			{
				if(is_odd(i,j)) glBindTexture(GL_TEXTURE_2D, textures[WHITE_BISHOP_WHITE]);
				else glBindTexture(GL_TEXTURE_2D, textures[WHITE_BISHOP_BLACK]);
			}
			else if(board[cell] == WHITE_KNIGHT1 || board[cell] == WHITE_KNIGHT2)
			{
				if(is_odd(i,j)) glBindTexture(GL_TEXTURE_2D, textures[WHITE_KNIGHT_WHITE]);
				else glBindTexture(GL_TEXTURE_2D, textures[WHITE_KNIGHT_BLACK]);
			}
			else if(board[cell] == WHITE_ROOK1 || board[cell] == WHITE_ROOK2)
			{
				if(is_odd(i,j)) glBindTexture(GL_TEXTURE_2D, textures[WHITE_ROOK_WHITE]);
				else glBindTexture(GL_TEXTURE_2D, textures[WHITE_ROOK_BLACK]);
			}
			else if(board[cell] == WHITE_PAWN1 || board[cell] == WHITE_PAWN2 || board[cell] == WHITE_PAWN3 || board[cell] == WHITE_PAWN4 || board[cell] == WHITE_PAWN5 || board[cell] == WHITE_PAWN6 || board[cell] == WHITE_PAWN7 || board[cell] == WHITE_PAWN8)
			{
				if(is_odd(i,j)) glBindTexture(GL_TEXTURE_2D, textures[WHITE_PAWN_WHITE]);
				else glBindTexture(GL_TEXTURE_2D, textures[WHITE_PAWN_BLACK]);	
			}

			if(board[cell] != NONE) draw_cell(i,j);
			glDisable(GL_TEXTURE_2D);
			glPopMatrix();
		}
	} //draw pieces

	//Draw outlined cell
	glPushMatrix();
		translate_to_cell(selected_cell);
		glScalef(cell_size+10, cell_size+10, 0);
		glLineWidth(5.0);
		glColor3f(0,1,0);
		glutWireCube(1);
		glLineWidth(1.0);
	glPopMatrix();

	if(mode == TUTORIAL_MODE)
	{
		//Draw moves
		for(int i=0;i<moves.move_count();i++)
		{
			Move current = moves.moves[i];
			for(int j=0;j<(int)current.cells.size();j++)
			{
				glPushMatrix();
					if(i == selected_move)
					{
						glColor3f(0,0,0);
					
						translate_to_cell(current.cells[j]);
						glPushMatrix();
						glScalef(6,6,0);
						glutSolidSphere(1,6,6);
						glPopMatrix();

						glPushMatrix();
						glScalef(cell_size+10, cell_size+10, 0);
						glLineWidth(5.0);
						glutWireCube(1);
						glLineWidth(1.0);
						glPopMatrix();
					}
					else
					{
						glColor3f(0,0,0);
						translate_to_cell(current.cells[j]);
						glScalef(6,6,0);
						glutSolidSphere(1,6,6);
					}
				glPopMatrix();
			}
		}
	}
	else if(mode == NORMAL_MODE)
	{
		//draw selected cell 2
		if(selected_cell_2 != -1)
		{
			glPushMatrix();
			translate_to_cell(selected_cell_2);
				glPushMatrix();
				glColor3f(0,0,0);
				glScalef(6,6,0);
				glutSolidSphere(1,6,6);
				glPopMatrix();

				glPushMatrix();
				glScalef(cell_size+10, cell_size+10, 0);
				glLineWidth(5.0);
				glutWireCube(1);
				glLineWidth(1.0);
				glPopMatrix();
			glPopMatrix();
		}
	}

	if(isFirstTime)
	{
		query_move();
		isFirstTime = false;
	}
}

void draw_letter(int i)
{
	glPushMatrix();
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f);
		glVertex2f(cell_size*i-cell_size/4+offset,0);
		
		glTexCoord2f(1.0f, 0.0f);
		glVertex2f(cell_size*i+cell_size/4+offset,0);
		
		glTexCoord2f(1.0f, 1.0f);
		glVertex2f(cell_size*i+cell_size/4+offset,cell_size/4);
		
		glTexCoord2f(0.0f, 1.0f);
		glVertex2f(cell_size*i-cell_size/4+offset,cell_size/4);
	glEnd();
	glPopMatrix();
}

void draw_number(int i)
{
	glPushMatrix();
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f);
		glVertex2f(0, (cell_size*i+offset-cell_size/4));
		
		glTexCoord2f(0.0f, 1.0f);
		glVertex2f(0, (cell_size*i+offset+cell_size/4));
		
		glTexCoord2f(1.0f, 1.0f);
		glVertex2f(cell_size/4, (cell_size*i+offset+cell_size/4));
		
		glTexCoord2f(1.0f, 0.0f);
		glVertex2f(cell_size/4, (cell_size*i+offset-cell_size/4));
	glEnd();
	glPopMatrix();
}

//Draws a cell with a piece on it (called after the appropriate texture is set)
void draw_cell(int i, int j)
{
	glPushMatrix();
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f);
		glVertex2f(cell_size*i-cell_size/2+offset,height-(cell_size*j+offset+cell_size/2));
		
		glTexCoord2f(1.0f, 0.0f);
		glVertex2f(cell_size*i+cell_size/2+offset,height-(cell_size*j+offset+cell_size/2));
		
		glTexCoord2f(1.0f, 1.0f);
		glVertex2f(cell_size*i+cell_size/2+offset,height-(cell_size*j+offset-cell_size/2));
		
		glTexCoord2f(0.0f, 1.0f);
		glVertex2f(cell_size*i-cell_size/2+offset,height-(cell_size*j+offset-cell_size/2));
	glEnd();
	glPopMatrix();
}

bool is_odd(int i, int j)
{
	if(j%2==0)
	{
		if(i%2==0) return false;
		else return true;
	}
	else
	{
		if(i%2==0) return true;
		else return false;
	}
}

//Translates the matrix stack to center of cell
void translate_to_cell(int id)
{
	int row = (id - id%8)/8;
	int column = id - (row*8);
	translate_to_cell(column,row);
}

//Translates the matrix stack to center of cell
void translate_to_cell(int i, int j)
{
	glTranslatef(cell_size*i, height-(cell_size*j+2*offset), 0);
	glTranslatef(offset, offset, 0);
}

void set_moves()
{
	int type = -1;
	if(board[selected_cell] == WHITE_KING || board[selected_cell] == BLACK_KING) type = KING;
	else if(board[selected_cell] == WHITE_QUEEN || board[selected_cell] == BLACK_QUEEN) type = QUEEN;
	else if(board[selected_cell] == WHITE_BISHOP1 || board[selected_cell] == WHITE_BISHOP2 || board[selected_cell] == BLACK_BISHOP1 || board[selected_cell] == BLACK_BISHOP2) type = BISHOP;
	else if(board[selected_cell] == WHITE_KNIGHT1 || board[selected_cell] == WHITE_KNIGHT2 || board[selected_cell] == BLACK_KNIGHT1 || board[selected_cell] == BLACK_KNIGHT2) type = KNIGHT;
	else if(board[selected_cell] == WHITE_ROOK1 || board[selected_cell] == WHITE_ROOK2 || board[selected_cell] == BLACK_ROOK1 || board[selected_cell] == BLACK_ROOK2) type = ROOK;
	else if(board[selected_cell] == WHITE_PAWN1 || board[selected_cell] == WHITE_PAWN2 || board[selected_cell] == WHITE_PAWN3 || board[selected_cell] == WHITE_PAWN4 ||board[selected_cell] == WHITE_PAWN5 || board[selected_cell] == WHITE_PAWN6 || board[selected_cell] == WHITE_PAWN7 || board[selected_cell] == WHITE_PAWN8) type = PAWN;
	else if(board[selected_cell] == BLACK_PAWN1 || board[selected_cell] == BLACK_PAWN2 || board[selected_cell] == BLACK_PAWN3 || board[selected_cell] == BLACK_PAWN4 ||board[selected_cell] == BLACK_PAWN5 || board[selected_cell] == BLACK_PAWN6 || board[selected_cell] == BLACK_PAWN7 || board[selected_cell] == BLACK_PAWN8) type = PAWN;

	if(type == -1){ reset_selected(); }
	else{ moves = Move_Utility::determine_move(type,selected_cell,board); show_moves = true; }
	
	glutPostRedisplay();
}

void select_move()
{
	selected_move++;
	if(selected_move >= moves.move_count()) selected_move = 0;

	wcscpy(string_to_speak, L"");
	speak_cell(selected_cell, false, string_to_speak, is_white_move, false);
	wcscat(string_to_speak, L" to ");
	Move m = moves.moves[selected_move];
	speak_cell(m.cells[m.cells.size()-1], true, string_to_speak, is_white_move, false);
}

void query_move()
{
	if(is_white_move) white_voice->Speak(L"Blue has the move", SPF_ASYNC|SPF_PURGEBEFORESPEAK , NULL);
	else black_voice->Speak(L"Red has the move", SPF_ASYNC|SPF_PURGEBEFORESPEAK , NULL);
}

void perform_move(int id)
{
	
	//If the same cell
	if(mode == NORMAL_MODE && selected_cell_2 == selected_cell)
	{
		selected_cell_2 = -1;
		reset_selected();
	}
		
	//Ensure that the piece being moved is proper color
	if(is_white_move && board[id] >= BLACK_KING && board[id] <= BLACK_PAWN8)
	{
		white_voice->Speak(L"That is not your piece", SPF_ASYNC|SPF_PURGEBEFORESPEAK , NULL);
		return;
	}

	if(!is_white_move && board[id] >= WHITE_KING && board[id] <= WHITE_PAWN8)
	{
		black_voice->Speak(L"That is not your piece", SPF_ASYNC|SPF_PURGEBEFORESPEAK , NULL);
		return;
	}
	
	if(mode == TUTORIAL_MODE)
	{
		Move m = moves.moves[selected_move];

		wcscpy(string_to_speak, L"");
		if(is_white_move) wcscat(string_to_speak, L"Blue Moves ");
		else wcscat(string_to_speak, L"Red Moves ");
		speak_cell(id, false, string_to_speak, is_white_move, true);	
		wcscat(string_to_speak, L" to ");
		speak_cell(m.cells[m.cells.size()-1], true, string_to_speak, is_white_move, true);

		board[m.cells[(int)m.cells.size()-1]] = board[id];
		board[id] = NONE;
		is_white_move = !is_white_move;
		selected_cell = m.cells[(int)m.cells.size()-1];
		reset_selected();
	
		mode = TUTORIAL_MODE;
	}
	else if(mode == NORMAL_MODE)
	{
		//Confirm that move is legit
		bool legit = false;
		for(int i=0;i<moves.move_count();i++)
		{
			Move current = moves.moves[i];
			if(selected_cell_2 == current.cells[current.cells.size()-1])
			{
				legit = true;
				break;
			}
		}
		
		if(legit)
		{
			wcscpy(string_to_speak, L"");
			if(is_white_move) wcscat(string_to_speak, L"Blue Moves ");
			else wcscat(string_to_speak, L"Red Moves ");
			speak_cell(id, false, string_to_speak, is_white_move, true);	
			wcscat(string_to_speak, L" to ");
			speak_cell(selected_cell_2, true, string_to_speak, is_white_move, true);

			board[selected_cell_2] = board[id];
			board[id] = NONE;
			is_white_move = !is_white_move;
			selected_cell = selected_cell_2;
			selected_cell_2 = -1;
			reset_selected();
		}
		else
		{
			if(is_white_move) white_voice->Speak(L"Error", SPF_ASYNC|SPF_PURGEBEFORESPEAK , NULL);
			else black_voice->Speak(L"Error", SPF_ASYNC|SPF_PURGEBEFORESPEAK , NULL);
		}

		mode = NORMAL_MODE;
	}

	glutPostRedisplay();
}

void reset_selected()
{
	show_moves = false;
	selected_move = -1;
	moves = Move_Envelope();
}

bool init_voices()
{
	//Initialize COM object
    if (FAILED(::CoInitialize(NULL))) return false;

	//Setup voices
	HRESULT black_hr = CoCreateInstance(CLSID_SpVoice, NULL, CLSCTX_ALL, IID_ISpVoice, (void **)&black_voice);
	HRESULT white_hr = CoCreateInstance(CLSID_SpVoice, NULL, CLSCTX_ALL, IID_ISpVoice, (void **)&white_voice);
	
	if(SUCCEEDED (black_hr) ) 
	{		
		CComPtr<ISpObjectToken> cpTokenEng;
		SpFindBestToken(SPCAT_VOICES, L"Gender=Female", NULL, &cpTokenEng);

		black_voice->SetVoice(cpTokenEng);
		black_voice->Speak(L"Red Initialized", 0, NULL);
	}

	if(SUCCEEDED (white_hr) )
	{
		CComPtr<ISpObjectToken> cpTokenEng;
		SpFindBestToken(SPCAT_VOICES, L"Gender=Male", NULL, &cpTokenEng);
		
		white_voice->SetVoice(cpTokenEng);
		white_voice->Speak(L"Blue Initialized", 0, NULL);
	}

    ::CoUninitialize();
	return true;
}

void speak(WCHAR theString[30], bool use_white_voice)
{
	if(use_white_voice) white_voice->Speak(theString, SPF_ASYNC|SPF_PURGEBEFORESPEAK , NULL);
	else black_voice->Speak(theString, SPF_ASYNC|SPF_PURGEBEFORESPEAK , NULL);
}

void speak_cell(int id, bool talk, WCHAR *theString, bool use_white_voice, bool isCapture)
{
	speak_cell_location(id, theString);
	speak_cell_piece(id, theString);

	if(board[id] >= BLACK_KING && board[id] <= BLACK_PAWN8 && !isCapture) use_white_voice = false; 
	else if(board[id] >= WHITE_KING && board[id] <= WHITE_PAWN8 && !isCapture) use_white_voice = true; 

	if(talk) speak(theString, use_white_voice);
}

void speak_cell_location(int id, WCHAR *theString)
{
	int row = (id - id%8)/8;
	int column = id - (row*8);

	//The column
	if(column == 0)			wcscat( theString, L"a" );
	else if(column == 1)	wcscat( theString, L"b" );
	else if(column == 2)	wcscat( theString, L"c" );
	else if(column == 3)	wcscat( theString, L"d" );
	else if(column == 4)	wcscat( theString, L"e" );
	else if(column == 5)	wcscat( theString, L"f" );
	else if(column == 6)	wcscat( theString, L"g" );
	else if(column == 7)	wcscat( theString, L"h" );

	//The row
	if(row == 0)		wcscat( theString, L"8 " );
	else if(row == 1)	wcscat( theString, L"7 " );
	else if(row == 2)	wcscat( theString, L"6 " );
	else if(row == 3)	wcscat( theString, L"5 " );
	else if(row == 4)	wcscat( theString, L"4 " );
	else if(row == 5)	wcscat( theString, L"3 " );
	else if(row == 6)	wcscat( theString, L"2 " );
	else if(row == 7)	wcscat( theString, L"1 " );
}

void speak_cell_piece(int id, WCHAR *theString)
{
	//The piece type
	if(board[id] == NONE)	wcscat( theString, L"Empty");
	else if(board[id] == WHITE_KING || board[id] == BLACK_KING)	wcscat( theString, L"King");
	else if(board[id] == WHITE_QUEEN || board[id] == BLACK_QUEEN)	wcscat( theString, L"Queen");
	else if(board[id] == WHITE_BISHOP1 || board[id] == WHITE_BISHOP2 || board[id] == BLACK_BISHOP1 || board[id] == BLACK_BISHOP2)	wcscat( theString, L"Bishop");
	else if(board[id] == WHITE_KNIGHT1 || board[id] == WHITE_KNIGHT2 || board[id] == BLACK_KNIGHT1 || board[id] == BLACK_KNIGHT2)	wcscat( theString, L"Knight");
	else if(board[id] == WHITE_ROOK1 || board[id] == WHITE_ROOK2 || board[id] == BLACK_ROOK1 || board[id] == BLACK_ROOK2)	wcscat( theString, L"Rook");
	else if(board[id] == WHITE_PAWN1 || board[id] == WHITE_PAWN2 || board[id] == WHITE_PAWN3 || board[id] == WHITE_PAWN4 || board[id] == WHITE_PAWN5 || board[id] == WHITE_PAWN6 || board[id] == WHITE_PAWN7 || board[id] == WHITE_PAWN8)	wcscat( theString, L"Pawn");
	else if(board[id] == BLACK_PAWN1 || board[id] == BLACK_PAWN2 || board[id] == BLACK_PAWN3 || board[id] == BLACK_PAWN4 || board[id] == BLACK_PAWN5 || board[id] == BLACK_PAWN6 || board[id] == BLACK_PAWN7 || board[id] == BLACK_PAWN8)	wcscat( theString, L"Pawn");
}

/*----------------------------------------------------------------------------
 * OpenGL Functions
 *----------------------------------------------------------------------------*/
void gl_init()
{
	glClearColor(0,0,0.5,0);
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0,850,0,850);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void gl_draw()
{
	glClear(GL_COLOR_BUFFER_BIT);
	if(!simulate_blindness)draw_board();
	glutSwapBuffers();
}

void gl_special(int key, int x, int y)
{
	if(mode == TUTORIAL_MODE)
	{
		switch(key)
		{
			case GLUT_KEY_UP: if(selected_cell > 7) selected_cell-=8; break;
			case GLUT_KEY_DOWN: if(selected_cell < 56) selected_cell+=8; break;
			case GLUT_KEY_LEFT: if(selected_cell%8 != 0) selected_cell--; break;
			case GLUT_KEY_RIGHT: if((selected_cell+1)%8 != 0) selected_cell++; break;
		}

		wcscpy(string_to_speak, L"");
		speak_cell(selected_cell, true, string_to_speak, is_white_move, false);	
		reset_selected();
	}
	else if(mode == NORMAL_MODE)
	{	
		//no cell selected to move
		if(selected_cell_2 == -1)
		{
			switch(key)
			{
				case GLUT_KEY_UP: if(selected_cell > 7) selected_cell-=8; break;
				case GLUT_KEY_DOWN: if(selected_cell < 56) selected_cell+=8; break;
				case GLUT_KEY_LEFT: if(selected_cell%8 != 0) selected_cell--; break;
				case GLUT_KEY_RIGHT: if((selected_cell+1)%8 != 0) selected_cell++; break;
			}

			wcscpy(string_to_speak, L"");
			speak_cell(selected_cell, true, string_to_speak, is_white_move, false);	
			reset_selected();
		}
		//if a cell is already selected -- move second selection square
		else
		{
			switch(key)
			{
				case GLUT_KEY_UP: if(selected_cell_2 > 7) selected_cell_2-=8; break;
				case GLUT_KEY_DOWN: if(selected_cell_2 < 56) selected_cell_2+=8; break;
				case GLUT_KEY_LEFT: if(selected_cell_2%8 != 0) selected_cell_2--; break;
				case GLUT_KEY_RIGHT: if((selected_cell_2+1)%8 != 0) selected_cell_2++; break;
			}
	
			wcscpy(string_to_speak, L"");
			speak_cell(selected_cell_2, true, string_to_speak, is_white_move, false);	
		}
	}

	glutPostRedisplay();
}

void gl_key(unsigned char key, int x, int y)
{
	switch(key)
	{
		case ' ': 
			if(mode == TUTORIAL_MODE)
			{
				if(!show_moves) set_moves(); 
				else select_move();
			}
			else if(mode == NORMAL_MODE)
			{
				set_moves();
				selected_cell_2 = selected_cell;
			}
			break;
		case '\r':
			if(mode == TUTORIAL_MODE)
			{
				if(selected_move != -1) perform_move(selected_cell);
			}
			else if(mode == NORMAL_MODE)
			{
				if(selected_cell_2 != -1) perform_move(selected_cell);
			}
			break;
		case 'q':
			exit(0);
			break;
		case '/':
			query_move();
			break;
		case 'b':
			simulate_blindness = !simulate_blindness;
			break;
		case 27:
			if(mode == TUTORIAL_MODE)
			{
				reset_selected();
				speak(L"Entering manual mode", is_white_move);
				mode = NORMAL_MODE;
			}
			else if(mode == NORMAL_MODE)
			{
				selected_cell_2 = -1;
				reset_selected();
				speak(L"Entering tutorial mode", is_white_move);
				mode = TUTORIAL_MODE;
			}
			break;
		default:
			break;
	}

	glutPostRedisplay();
}

// From http://www.gametutorials.com/Tutorials/opengl/OpenGL_Pg2.htm#TextureMapping
void CreateTexture(UINT textureArray[], LPSTR strFileName, int textureID)
{
	AUX_RGBImageRec *pBitmap = NULL;
	
	if(!strFileName)									// Return from the function if no file name was passed in
		return;
	
	// We need to load the texture data, so we use a cool API that the glaux.lib offers.	
	pBitmap = auxDIBImageLoad(strFileName);				// Load the bitmap and store the data
	
	if(pBitmap == NULL)									// If we can't load the file, quit!
		exit(0);

	// Generate a texture with the associative texture ID stored in the array
	glGenTextures(1, &textureArray[textureID]);

	// Bind the texture to the texture arrays index and init the texture
	glBindTexture(GL_TEXTURE_2D, textureArray[textureID]);

	// Build Mipmaps (builds different versions of the picture for distances - looks better)
	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, pBitmap->sizeX, pBitmap->sizeY, GL_RGB, GL_UNSIGNED_BYTE, pBitmap->data);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR);

	//glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, pBitmap->sizeX, pBitmap->sizeY,0, GL_RGB, GL_UNSIGNED_BYTE, pBitmap->data); 

	// Now we need to free the bitmap data that we loaded since openGL stored it as a texture
	if (pBitmap)										// If we loaded the bitmap
	{
		if (pBitmap->data)								// If there is texture data
		{
			free(pBitmap->data);						// Free the texture data, we don't need it anymore
		}

		free(pBitmap);									// Free the bitmap structure
	}
}

int main(int argc, char** argv)
{ 	
	init_voices();

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowSize(width,height);
	glutInitWindowPosition(100,100);
	glutCreateWindow("ACT - Accessible Chess Tutor");

	gl_init();
	init_board();

	glutDisplayFunc(gl_draw);
	glutKeyboardFunc(gl_key);
	glutSpecialFunc(gl_special);
	glutMainLoop();
	return 0;
}